/*
  aes.cpp - AES Library for NXP-JN516x

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#include "aes.h"

AES::AESMODE AES::_mode;
tuAES_Block  AES::_nonce;

void AES::begin(AESMODE mode, uint8 *pu8KeyData, uint8 u8KeyLen)
{
  tsReg128 sReg128;
  _mode = mode;
  memset(&sReg128, 0, sizeof(sReg128));
  memcpy(&sReg128, pu8KeyData, u8KeyLen < sizeof(sReg128) ? u8KeyLen : sizeof(sReg128));
  bACI_WriteKey(&sReg128);
}

bool AES::encryption(uint8 *pau8Data, uint8 u8DataLen, uint32 u32Nonce, uint8 *pu8AuthData, uint8 u8AuthLen, uint8 *pau8Chksum)
{
  uint8 csum[32];
  if (!pau8Chksum)
    pau8Chksum = csum;
  _nonce.au32[0] = u32Nonce;
  vACI_OptimisedCcmStar(true, _mode, u8AuthLen, u8DataLen, &_nonce, pu8AuthData, pau8Data, pau8Chksum, 0);
  return true;
}

bool AES::decryption(uint8 *pau8Data, uint8 u8DataLen, uint32 u32Nonce, uint8 *pu8AuthData, uint8 u8AuthLen, uint8 *pau8Chksum)
{
  bool_t bVerify = false;
  _nonce.au32[0] = u32Nonce;
  vACI_OptimisedCcmStar(false, _mode, u8AuthLen, u8DataLen, &_nonce, pu8AuthData, pau8Data, pau8Chksum, &bVerify);
  return bVerify;
}

AES::AESMODE AES::mode(void)
{
  return _mode;
}
