/*
  alarm.h - Alarm Library for NXP-JN516x

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#pragma once

#include <jendefs.h>
#include <AppHardwareApi.h>
#include "timer.h"

class Alarm
{
  public:

    typedef bool (*handler_t)(Alarm& alarm);

    Alarm& interval(int64 interval, bool fromnow = true);
    int64 interval(void);
    Alarm& handler(handler_t handler);
    handler_t handler(void);
    Alarm& param(uint8 num, void *val);
    void *param(uint8 num);
    bool expire(void);

  protected:

    friend class AlarmClass;

    Alarm    *_next;
    uint64    _alarm;
    int64     _interval;
    handler_t _handler;
    void     *_params[4];
};

class AlarmClass
{
  public:

    void begin(void);
    void begin(Timer::TIMER timer);
    bool start(Alarm& alarm);
    void cancel(Alarm& alarm);
    void cancel(uint8 num, void *arg);

  protected:

    bool add(Alarm& alarm);
    void restart(int64 interval);

  private:

    void timeup(void);

    static void TimerCallback(uint32 u32Device, uint32 u32ItemBitmap);

    Timer::TIMER _timer;
    Alarm *_alarm_list;
};

extern AlarmClass AlarmTimer;
