/*
  analog.h - Analog Configure Library for NXP-JN516x

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#pragma once

#include <jendefs.h>
#include <AppHardwareApi.h>

/*
   I/O Ports

     External VREF: ADC2-VREF
*/

#define ANALOG_BANDGAP  1235 // mV

class Analog
{
  public:

    typedef enum
    {
      CAPTURE_2 = E_AHI_AP_SAMPLE_2, // 2 clock periods
      CAPTURE_4 = E_AHI_AP_SAMPLE_4, // 4 clock periods
      CAPTURE_6 = E_AHI_AP_SAMPLE_6, // 6 clock periods
      CAPTURE_8 = E_AHI_AP_SAMPLE_8, // 8 clock periods
    } CAPTURE;

    typedef enum
    {
      CLOCK_2MHZ   = E_AHI_AP_CLOCKDIV_2MHZ,   // divisor of 8
      CLOCK_1MHZ   = E_AHI_AP_CLOCKDIV_1MHZ,   // divisor of 16
      CLOCK_500KHZ = E_AHI_AP_CLOCKDIV_500KHZ, // divisor of 32 (500kHz is recommended for ADC)
      CLOCK_250KHZ = E_AHI_AP_CLOCKDIV_250KHZ, // divisor of 64
    } CLOCK;

    static void begin(void);
    static void configure(void);
    static void stop(void);
    static void decoupling(bool enable);
    static void interrupt(bool enable);
    static void clock(CLOCK clock);
    static void capture(CAPTURE capture);
    static void reference(bool external);

  private:

    static bool    _interrupt;
    static CLOCK   _clock;
    static CAPTURE _capture;
    static bool    _reference;
};
