/*
  board_monostick.h - Board Library for MONOWIRESS MONOSTICK

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#pragma once

#include "gpio.h"
#include "led.h"

class Board
{
  public:

    static const Gpio::PIN PIN_LED_R   = Gpio::DIO16;
    static const Gpio::PIN PIN_LED_Y   = Gpio::DO1;
    static const Gpio::PIN PIN_WDT_EN  = Gpio::DIO11;
    static const Gpio::PIN PIN_WDT_OUT = Gpio::DIO9;

    static
    void begin(void)
    {
      /* Setup LED */
      Led::setup(0, PIN_LED_R, HIGH);
      Led::setup(1, PIN_LED_Y, HIGH);
      /* Setup GPIO */
      Gpio::digitalWrite(PIN_WDT_EN, HIGH);
      Gpio::pinMode(PIN_WDT_EN , Gpio::OUTPUT);
      Gpio::pinMode(PIN_WDT_OUT, Gpio::OUTPUT);
      watchdog(true);
    }

    static
    void watchdog(bool enable)
    {
      Gpio::digitalWrite(PIN_WDT_EN, !enable);
    }

    static
    void handle(void)
    {
      Gpio::digitalWrite(PIN_WDT_OUT, !Gpio::digitalRead(PIN_WDT_OUT));
    }
};
