/*
  clock.h - Clock Library for NXP-JN516x

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#pragma once

#include <jendefs.h>
#include <AppHardwareApi.h>

/*
   I/O Ports

    External RC  : DIO9
    External XTAL: DIO9 and DIO10
*/

class Clock
{
  public:

    typedef enum
    {
      EXT32KSRC_RC   = E_AHI_EXTERNAL_RC, // external RC
      EXT32KSRC_XTAL = E_AHI_XTAL,        // external crystal
    } EXT32KSRC;

    static bool external32K(EXT32KSRC clksrc);
    static void external32KXtal(void);

    typedef enum
    {
      EXT32KRC_530nA = 2, // +-300ppm (default)
      EXT32KRC_350nA = 3, // +-600ppm
    } EXT32KRC;

    static void external32KRC(EXT32KRC trim);

    typedef enum
    {
      CLKSRC_XTAL = E_AHI_XTAL_ON,
      CLKSRC_RC   = E_AHI_XTAL_OFF,
    } CLKSRC;

    static void source(CLKSRC clksrc, bool powerdown = false);
    static CLKSRC source(void);

    typedef enum
    {
      //                           32MHz (27MHz)
      CLKDIV8  = E_AHI_XTAL_4MHZ,  //  4 ( 3.38) MHz
      CLKDIV4  = E_AHI_XTAL_8MHZ,  //  8 ( 6.75) MHz
      CLKDIV2  = E_AHI_XTAL_16MHZ, // 16 (13.5 ) MHz
      CLKDIV1  = E_AHI_XTAL_32MHZ, // 32 (27   ) MHz
      CLKDIV32 = E_AHI_XTAL_1MHZ,  //  1 ( 0.84) MHz
      CLKDIV16 = E_AHI_XTAL_2MHZ,  //  2 ( 1.69) MHz
    } CLKDIV;

    static bool rate(CLKDIV clkdiv);
    static CLKDIV rate(void);
    static bool external32MXtal(void);

    typedef enum
    {
      XTALPULL_NONE = E_AHI_XTAL_PULL_NONE,  // No additional crystal load capacitance (default)
      XTALPULL_50   = E_AHI_XTAL_PULL_50PC,  // 1pF
      XTALPULL_100  = E_AHI_XTAL_PULL_100PC, // 2pF
    } XTALPULL;

    static void xtalPull(XTALPULL mode);
    static void fastStartUp(bool automatic, bool powerdown);
    static bool internal32MRC(void);
    static void optimizeWaitStates(void);
};
