/*
  comparator.cpp - Comparator Library for NXP-JN516x

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#include "sysctrl.h"
#include "analog.h"
#include "gpio.h"
#include "comparator.h"

void Comparator::begin(SEL sel, HYST hyst, PR_HWINT_APPCALLBACK prCompCallback, bool rising)
{
  switch (sel)
  {
    case SEL_EXT:
    case SEL_EXT_INVERSE:
      Gpio::pinMode(Gpio::DIO16, Gpio::INPUT);
      Gpio::pinMode(Gpio::DIO17, Gpio::INPUT);
      break;
    case SEL_BANDGAP:
    case SEL_BANDGAP_INVERSE:
      Analog::configure();
      Gpio::pinMode(sel == SEL_BANDGAP ? Gpio::DIO16 : Gpio::DIO17, Gpio::INPUT);
      break;
  }
  SysCtrl::interrupt(SysCtrl::INTSRC_COMP0, prCompCallback);
  vAHI_ComparatorIntEnable(E_AHI_AP_COMPARATOR_1, prCompCallback != 0, rising);
  vAHI_ComparatorEnable(E_AHI_AP_COMPARATOR_1, hyst, sel);
}

void Comparator::end(void)
{
  vAHI_ComparatorDisable(E_AHI_AP_COMPARATOR_1);
}

void Comparator::lowPowerMode(bool enable)
{
  vAHI_ComparatorLowPowerMode(enable);
}

bool Comparator::status(void)
{
  return (u8AHI_ComparatorStatus() & E_AHI_AP_COMPARATOR_MASK_1) != 0;
}
