/*
  comparator.h - Comparator Library for NXP-JN516x

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#pragma once

#include <jendefs.h>
#include <AppHardwareApi.h>


/*
   I/O Ports

     COMP1P: DIO16
     COMP1M: DIO17

     Input Signal Reference Signal Enumeration (u8SignalSelect)
     ------------------------------------------------------------
     COMP1P       COMP1M           E_AHI_COMP_SEL_EXT
     COMP1P       Vref             E_AHI_COMP_SEL_BANDGAP
     COMP1M       COMP1P           E_AHI_COMP_SEL_EXT_INVERSE
     COMP1M       Vref             E_AHI_COMP_SEL_BANDGAP_INVERSE
*/

class Comparator
{
  public:

    // u8Hysteresis Hysteresis setting (controllable from 0 to 40mV)
    typedef enum
    {
      HYST_0MV  = E_AHI_COMP_HYSTERESIS_0MV,  // 0mV
      HYST_10MV = E_AHI_COMP_HYSTERESIS_10MV, // 10mV
      HYST_20MV = E_AHI_COMP_HYSTERESIS_20MV, // 20mV
      HYST_40MV = E_AHI_COMP_HYSTERESIS_40MV, // 40mV
    } HYST;

    // u8SignalSelect Selection of input and reference signals (see table above):
    typedef enum
    {
      SEL_EXT             = E_AHI_COMP_SEL_EXT,
      SEL_BANDGAP         = E_AHI_COMP_SEL_BANDGAP,
      SEL_EXT_INVERSE     = E_AHI_COMP_SEL_EXT_INVERSE,
      SEL_BANDGAP_INVERSE = E_AHI_COMP_SEL_BANDGAP_INVERSE,
    } SEL;

    static void begin(SEL sel, HYST hyst = HYST_20MV, PR_HWINT_APPCALLBACK prCompCallback = 0, bool rising = false);
    static void end(void);
    static void lowPowerMode(bool enable);
    static bool status(void);
};
