/*
  debug.h - Debug Utility for NXP-JN516x

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#pragma once

#include <stdarg.h>
#include <jendefs.h>
#include "uart.h"
#include "task.h"

#ifndef DEBUG_UART
#define DEBUG_UART UART_0
#endif

class Debug
{
  public:

    static void begin(Uart::UART uart = Uart::DEBUG_UART);
    static void printf(const char *format, ...);
    static void putch(char c);
    static void mdump(const uint8 *addr, uint16 len);

  private:

    typedef struct
    {
      char   fillchar;
      char   alignchar;
      char   signchar;
      char   hexchar;
      uint8  width;
      uint8  precision;
      uint8  typelong;
      uint8  base;
      uint64 value;
    } format_t;

    static void putstr(format_t& option, const char *str);
    static void nformat(format_t& option);

    static Uart *_output;
    static Mutex _mutex;
};
