/*
  eeprom.h - EEPROM Library for NXP-JN516x

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#pragma once

#include <string.h>
#include <jendefs.h>
#include <AppHardwareApi.h>

/*
JN5161-001:  64kB Flash,  8kB RAM, 4 kB EEPROM suitable for IEEE802.15.4 and RF4CE applications
JN5164-001: 160kB Flash, 32kB RAM, 4 kB EEPROM suitable for Jennet-IP, IEEE802.15.4 and RF4CE applications
JN5168-001: 256kB Flash, 32kB RAM, 4 kB EEPROM suitable for all applications

Parameter              Min    Typ    Max Unit  Notes
-----------------------------------------------------------------------
Flash endurance         10K  100/50K     Cycle Program/erase
Flash erase time             100         ms    One or more sectors
Flash programming time       1.0         ms    256 bytes
EEPROM endurance       100K  1M/500K     °C    Program/erase (see 4.4 )
EEPROM erase time            1.8         ms    One 64 bytes page
EEPROM programming time      1.1         ms    Between 1 & 64 bytes
Retention time powered   10              Years Flash & EEPRO
unpowered                20
-----------------------------------------------------------------------
*/

class EEPRom
{
  public:

    static void begin(void);
    static bool erase(uint16 u16Offset = 0, uint16 u16Length = 4096);
    static bool write(uint16 u16Offset, uint8 *pu8Buffer, uint16 u16Length);
    static bool read(uint16 u16Offset, uint8 *pu8Buffer, uint16 u16Length);

  private:

    static uint8  _segbytes;
    static uint16 _segments;
};
