/*
  flash.cpp - Flash Library for NXP-JN516x

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#include "flash.h"

bool Flash::begin(teFlashChipType flashType, tSPIflashFncTable *pCustomFncTable, PR_HWINT_APPCALLBACK prFlashEECCallback)
{
  bAHI_FlashEECerrorInterruptSet(prFlashEECCallback != 0, prFlashEECCallback);
  return bAHI_FlashInit(flashType, pCustomFncTable);
}

bool Flash::erase(uint8 u8Sector)
{
  return bAHI_FlashEraseSector(u8Sector);
}

bool Flash::program(uint32 u32Addr, uint16 u16Len, uint8 *pu8Data)
{
  return bAHI_FullFlashProgram(u32Addr, u16Len, pu8Data);
}

bool Flash::read(uint32 u32Addr, uint16 u16Len, uint8 *pu8Data)
{
  return bAHI_FullFlashRead(u32Addr, u16Len, pu8Data);
}

void Flash::power(bool enable)
{
  if (enable)
    vAHI_FlashPowerUp();
  else
    vAHI_FlashPowerDown();
}

void Flash::TemperatureOperation(bool enable)
{
  vAHI_ExtendedTemperatureOperation(enable);
}
