/*
  flash.h - Flash Library for NXP-JN516x

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#pragma once

#include <jendefs.h>
#include <AppHardwareApi.h>

/*
JN5161-001:  64kB Flash,  8kB RAM, 4 kB EEPROM suitable for IEEE802.15.4 and RF4CE applications
JN5164-001: 160kB Flash, 32kB RAM, 4 kB EEPROM suitable for Jennet-IP, IEEE802.15.4 and RF4CE applications
JN5168-001: 256kB Flash, 32kB RAM, 4 kB EEPROM suitable for all applications
*/

class Flash
{
  public:

    static const uint16 SECTOR_SIZE = (32 * 1024);

    static bool begin(teFlashChipType flashType = E_FL_CHIP_AUTO, tSPIflashFncTable *pCustomFncTable = NULL, PR_HWINT_APPCALLBACK prFlashEECCallback = 0);
    //
    // 32K Sectors / 16-byte boundary
    //
    static bool erase(uint8 u8Sector);
    static bool program(uint32 u32Addr, uint16 u16Len, uint8 *pu8Data);
    static bool read(uint32 u32Addr, uint16 u16Len, uint8 *pu8Data);
    //
    // for external Flash memory device
    //
    static void power(bool enable);
    static void TemperatureOperation(bool enable);
};
