/*
  gpio.cpp - GPIO Library for NXP-JN516x

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#include "gpio.h"

uint32 Gpio::_doout;

void Gpio::begin(void)
{
  static const PIN pins[] =
  {
    DIO0 , DIO1 , DIO2 , DIO3 , DIO4 , DIO5 , DIO6 , DIO7 ,
    DIO8 , DIO9 , DIO10, DIO11, DIO12, DIO13, DIO14, DIO15,
    DIO16, DIO17, DIO18, DIO19, DO0  , DO1
  };
  for (uint8 i = 0; i < sizeof(pins) / sizeof(pins[0]); ++i)
    pinMode(pins[i], INPUT_PULLUP);
}

void Gpio::pinMode(PIN pin, MODE mode)
{
  uint32 bit = 1 << (pin & 0x1F);
  switch (pin >> 5)
  {
    case 0:
      if (mode == INPUT_PULLUP)
        vAHI_DioSetPullup(bit, 0);
      else
        vAHI_DioSetPullup(0, bit);
      if (mode == OUTPUT)
        vAHI_DioSetDirection(0, bit);
      else
        vAHI_DioSetDirection(bit, 0);
      break;
    case 1:
      if (mode == INPUT_PULLUP)
        vAHI_DoSetPullup(bit, 0);
      else
        vAHI_DoSetPullup(0, bit);
      bAHI_DoEnableOutputs(mode == OUTPUT); // DO0 and DO1
      break;
  }
}

void Gpio::digitalWrite(PIN pin, uint32 value)
{
  uint32 bit = 1 << (pin & 0x1F);
  switch (pin >> 5)
  {
    case 0:
      if (value)
        vAHI_DioSetOutput(bit, 0);
      else
        vAHI_DioSetOutput(0, bit);
      break;
    case 1:
      if (value)
      {
        _doout |= bit;
        vAHI_DoSetDataOut(bit, 0);
      }
      else
      {
        _doout &= ~bit;
        vAHI_DoSetDataOut(0, bit);
      }
      break;
  }
}

uint32 Gpio::digitalRead(PIN pin)
{
  switch (pin >> 5)
  {
    case 0:
      return (u32AHI_DioReadInput() >> pin) & 1;
    case 1:
      return (_doout >> (pin - 32)) & 1;
  }
  return 0;
}

void Gpio::wake(PIN pin, EDGE mode, PR_HWINT_APPCALLBACK prGpioCallback)
{
  uint32 bit = 1 << (pin & 0x1F);
  switch (pin >> 5)
  {
    case 0:
      if (prGpioCallback == 0)
        vAHI_DioWakeEnable(0, bit);
      else
      {
        if (pin <= DIO19)
          SysCtrl::interrupt((SysCtrl::INTSRC)pin, prGpioCallback);
        if (mode == RISING)
          vAHI_DioWakeEdge(bit, 0);
        else
          vAHI_DioWakeEdge(0, bit);
        vAHI_DioWakeEnable(bit, 0);
      }
      break;
  }
}

bool Gpio::wakeStatus(PIN pin)
{
  return (wakeStatus() >> pin) & 1;
}

uint32 Gpio::wakeStatus(void)
{
  return u32AHI_DioWakeStatus();
}
