/*
  gpio.h - GPIO Library for NXP-JN516x

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#pragma once

#include <jendefs.h>
#include <AppHardwareApi.h>
#include "sysctrl.h"

#define LOW  0
#define HIGH 1

class Gpio
{
  public:

    typedef enum
    {
      DIO0  = 0,
      DIO1  = 1,
      DIO2  = 2,
      DIO3  = 3,
      DIO4  = 4,
      DIO5  = 5,
      DIO6  = 6,
      DIO7  = 7,
      DIO8  = 8,
      DIO9  = 9,
      DIO10 = 10,
      DIO11 = 11,
      DIO12 = 12,
      DIO13 = 13,
      DIO14 = 14,
      DIO15 = 15,
      DIO16 = 16,
      DIO17 = 17,
      DIO18 = 18,
      DIO19 = 19,
      DO0   = 32,
      DO1   = 33,
    } PIN;

    typedef enum
    {
      INPUT,
      OUTPUT,
      INPUT_PULLUP,
    } MODE;

    static void begin(void);
    static void pinMode(PIN pin, MODE mode);
    static void digitalWrite(PIN pin, uint32 value);
    static uint32 digitalRead(PIN pin);

    typedef enum
    {
      RISING,
      FALLING,
    } EDGE;

    static void wake(PIN pin, EDGE mode, PR_HWINT_APPCALLBACK prGpioCallback = 0);
    static bool wakeStatus(PIN pin);
    static uint32 wakeStatus(void);

  private:

    static uint32 _doout;
};
