/*
  infrared.cpp - Infra-Red Transmitter Library for NXP-JN516x

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#include "timer.h"
#include "infrared.h"

bool InfraRedTransmitter::begin(PIN pin, bool bInvertOutput, uint16 period, FSC fsc, PR_HWINT_APPCALLBACK prInfraredCallback)
{
  if (prInfraredCallback)
    vAHI_InfraredRegisterCallback(prInfraredCallback);
  Timer::location(Timer::TIMER_2, pin & 1, pin & 2);
  Timer::DIOControl(Timer::TIMER_2, true);
  uint16 u16Lo = F_PCK / fsc;
  return bAHI_InfraredEnable(0, (uint32)u16Lo * 7 / 10, u16Lo, period * (F_PCK / 1000000) / u16Lo, bInvertOutput, prInfraredCallback != 0);
}

void InfraRedTransmitter::end(void)
{
  vAHI_InfraredDisable();
}

bool InfraRedTransmitter::start(uint32 *pu32BufferAddress, uint16 u16TransmissionLengthInBits)
{
  return bAHI_InfraredStart(pu32BufferAddress, u16TransmissionLengthInBits);
}

bool InfraRedTransmitter::status(void)
{
  return bAHI_InfraredStatus();
}
