/*
  infrared.h - Infra-Red Transmitter Library for NXP-JN516x

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#pragma once

#include <jendefs.h>
#include <AppHardwareApi.h>
#include "gpio.h"
/*
   I/O Ports

   Output: DIO12/DIO6
*/

#ifndef IR_DEFAULT_FSC
#define IR_DEFAULT_FSC FSC_38KHZ
#endif

class InfraRedTransmitter
{
  public:

    typedef enum
    {
      FSC_33KHZ = 33000,
      FSC_34KHZ = 34000,
      FSC_35KHZ = 35000,
      FSC_36KHZ = 36000,
      FSC_37KHZ = 37000,
      FSC_38KHZ = 38000,
      FSC_39KHZ = 39000,
      FSC_40KHZ = 40000,
    } FSC;

    typedef enum
    {
      PIN_DIO12 = 0,
      PIN_DIO6  = 1,
      PIN_DO0   = 2,
    } PIN;

    static bool begin(PIN pin = PIN_DIO12, bool bInvertOutput = false, uint16 period = 500, FSC fsc = FSC_38KHZ, PR_HWINT_APPCALLBACK prInfraredCallback = 0);
    static void end(void);
    static bool start(uint32 *pu32BufferAddress, uint16 u16TransmissionLengthInBits);
    static bool status(void);
};
