/*
  led.h - LED Library for NXP-JN516x

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#pragma once

#include "alarm.h"
#include "gpio.h"

#ifndef LED_BITS
#define LED_BITS 1
#endif
#define LED_COUNT (1 << LED_BITS)
#define LED_MASK  (LED_COUNT - 1)

class Led
{
  public:

    static void begin(void);
    static void setup(uint8 led, uint8 pin, uint8 off);
    static void off(uint8 led = 0, uint16 ms = 0);
    static void on(uint8 led = 0, uint16 ms = 0);
    static void toggle(uint8 led = 0, uint16 ms = 0);
    static bool status(uint8 led = 0);
    static bool expire(uint8 led = 0);
    static void handle(void);

  private:

    typedef enum
    {
      MODE_DISABLE,
      MODE_OFF,
      MODE_ON,
      MODE_TOGGLE,
    } MODE;

    typedef struct
    {
      uint8 pin;
      uint8 off;
      MODE  mode;
      bool  expire;
      Alarm alarm;
    } CTRL;

    static CTRL _leds[LED_COUNT];

    static void control(CTRL& ctrl, MODE mode, uint16 ms);
};
