/*
  mc3630_jn516x.h - MC3630 IO Library for NXP JN516x Series

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#pragma once

#include "spi.h"
#include "wire.h"
#include "system.h"
#include "mc3630.h"

class MC3630_IO
{
  public:

    static
    MC3630::IOMETHOD *begin(MC3630::IFMODE mode = MC3630::IFMODE_SPI, uint8_t slave = Spi::SEL_0)
    {
      _slave = slave;
      _mode  = mode;
      if (_mode & MC3630::IFMODE_I2C)
        Wire::begin(Wire::SPEED_800KHZ);
      else
      {
        _spiconfig.mode(SpiConfig::MODE_3);
        _spiconfig.speed(SpiConfig::SPEED_2MHZ);
      }
      _iomethod.writeIO           = writeIO;
      _iomethod.readIO            = readIO;
      _iomethod.setSpiHsModeIO    = setSpiHsModeIO;
      _iomethod.delayMicroseconds = delayMicroseconds;
      return &_iomethod;
    }

    static
    void writeIO(uint8_t cmd, const uint8_t *buf, uint8_t len)
    {
      if (_mode & MC3630::IFMODE_I2C)
        Wire::write(_slave, cmd, buf, len);
      else
      {
        _spiconfig.beginTransaction();
        Spi::write(_slave, cmd, buf, len);
        _spiconfig.endTransaction();
      }
    }

    static
    void readIO(uint8_t cmd, uint8_t *buf, uint8_t len)
    {
      if (_mode & MC3630::IFMODE_I2C)
        Wire::writeAndRead(_slave, &cmd, sizeof(cmd), buf, len);
      else
      {
        _spiconfig.beginTransaction();
        Spi::writeAndRead(_slave, &cmd, sizeof(cmd), buf, len);
        _spiconfig.endTransaction();
      }
    }

    static
    void setSpiHsModeIO(void)
    {
      _spiconfig.speed(SpiConfig::SPEED_8MHZ);
    };

    static
    void delayMicroseconds(uint32 us)
    {
      System::delayMicroseconds(us);
    }

  private:

    static MC3630::IFMODE   _mode;
    static MC3630::IOMETHOD _iomethod;
    static uint8_t          _slave;
    static SpiConfig        _spiconfig;
};
