/*
  muldiv.h - Multiplication and Division Library for NXP-JN516x

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#include "system.h"
#include "muldiv.h"

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wreturn-type"

int32 MulDiv::cmp64(int64 a, int64 b)
{
  // a=r3:r4, b=r5:r6
  asm volatile ("b.sub   r6, r6, r4");
  asm volatile ("b.addc  r3, r3, r0");
  asm volatile ("b.sub   r5, r5, r3");
  asm volatile ("b.movi  r3, 1");
  asm volatile ("b.blts  r5, r0, L_cmp64_1");
  asm volatile ("b.or    r3, r5, r6");
  asm volatile ("b.beq   r3, r0, L_cmp64_1");
  asm volatile ("b.movi  r3, -1");
  asm volatile ("L_cmp64_1:");
  // return r3
}

int32 MulDiv::cmpu64(uint64 a, uint64 b)
{
  // a=r3:r4, b=r5:r6
  asm volatile ("b.movi  r7, 1");
  asm volatile ("b.bgtu  r3, r5, L_cmpu64_2");
  asm volatile ("b.bne   r3, r5, L_cmpu64_1");
  asm volatile ("b.bgtu  r4, r6, L_cmpu64_2");
  asm volatile ("b.bne   r4, r6, L_cmpu64_1");
  asm volatile ("b.mov   r7, r0");
  asm volatile ("b.j     L_cmpu64_2");
  asm volatile ("L_cmpu64_1:");
  asm volatile ("b.movi  r7, -1");
  asm volatile ("L_cmpu64_2:");
  asm volatile ("b.mov   r3, r7");
  // return r3
}

int64 MulDiv::add64(int64 a, int64 b)
{
  // a=r3:r4, b=r5:r6
  asm volatile ("b.add   r4, r4, r6");
  asm volatile ("b.addc  r3, r3, r5");
  // return r3:r4
}

int64 MulDiv::sub64(int64 a, int64 b)
{
  // a=r3:r4, b=r5:r6
  asm volatile ("b.sub   r4, r4, r6");
  asm volatile ("b.addc  r5, r5, r0");
  asm volatile ("b.sub   r3, r3, r5");
  // return r3:r4
}

int64 MulDiv::shl64(int64 a, uint32 b)
{
  // a=r3:r4, b=r5
  asm volatile ("b.beq    r5, r0, L_shl64_3");
  asm volatile ("b.addi   r6, r0, 32");
  asm volatile ("b.bgeu   r5, r6, L_shl64_1");
  // b < 32
  asm volatile ("b.sub    r6, r6, r5");
  asm volatile ("b.sll    r3, r3, r5");
  asm volatile ("b.srl    r6, r4, r6");
  asm volatile ("b.or     r3, r3, r6");
  asm volatile ("b.sll    r4, r4, r5");
  asm volatile ("b.j      L_shl64_3");
  // b >= 32
  asm volatile ("L_shl64_1:");
  asm volatile ("b.sub    r5, r6, r5");
  asm volatile ("b.bgeu   r5, r6, L_shl64_2");
  asm volatile ("b.sll    r3, r4, r5");
  asm volatile ("b.mov    r4, r0");
  asm volatile ("b.j      L_shl64_3");
  // b >= 64
  asm volatile ("L_shl64_2:");
  asm volatile ("b.mov    r3, r0");
  asm volatile ("b.mov    r4, r0");
  // b == 0
  asm volatile ("L_shl64_3:");
  // return r3:r4
}

int64 MulDiv::shr64(int64 a, uint32 b)
{
  // a=r3:r4, b=r5
  asm volatile ("b.beq    r5, r0, L_shru64_3");
  asm volatile ("b.addi   r6, r0, 32");
  asm volatile ("b.bgeu   r5, r6, L_shru64_1");
  // b < 32
  asm volatile ("b.sub    r6, r6, r5");
  asm volatile ("b.srl    r4, r4, r5");
  asm volatile ("b.sll    r6, r3, r6");
  asm volatile ("b.or     r4, r4, r6");
  asm volatile ("b.srl    r3, r3, r5");
  asm volatile ("b.j      L_shru64_3");
  // b >= 32
  asm volatile ("L_shru64_1:");
  asm volatile ("b.sub    r5, r6, r5");
  asm volatile ("b.bgeu   r5, r6, L_shru64_2");
  asm volatile ("b.srl    r4, r3, r5");
  asm volatile ("b.mov    r3, r0");
  asm volatile ("b.j      L_shru64_3");
  // b >= 64
  asm volatile ("L_shru64_2:");
  asm volatile ("b.mov    r3, r0");
  asm volatile ("b.mov    r4, r0");
  // b == 0
  asm volatile ("L_shru64_3:");
  // return r3:r4
}

int64 MulDiv::sar64(int64 a, uint32 b)
{
  // a=r3:r4, b=r5
  asm volatile ("b.beq    r5, r0, L_shr64_3");
  asm volatile ("b.addi   r6, r0, 32");
  asm volatile ("b.bgeu   r5, r6, L_shr64_1");
  // b < 32
  asm volatile ("b.sub    r6, r6, r5");
  asm volatile ("b.srl    r4, r4, r5");
  asm volatile ("b.sll    r6, r3, r6");
  asm volatile ("b.or     r4, r4, r6");
  asm volatile ("b.sra    r3, r3, r5");
  asm volatile ("b.j      L_shr64_3");
  // b >= 32
  asm volatile ("L_shr64_1:");
  asm volatile ("b.sub    r5, r6, r5");
  asm volatile ("b.bgeu   r5, r6, L_shr64_2");
  asm volatile ("b.sra    r4, r3, r5");
  asm volatile ("b.srai   r3, r3, 31");
  asm volatile ("b.j      L_shr64_3");
  // b >= 64
  asm volatile ("L_shr64_2:");
  asm volatile ("b.srai   r4, r3, 31");
  asm volatile ("b.srai   r3, r3, 31");
  // b == 0
  asm volatile ("L_shr64_3:");
  // return r3:r4
}

uint64 MulDiv::mulu32u32(uint32 a, uint32 b)
{
  asm volatile ("b.mov  r5, %0" :: "r" (a) : "r5");
  asm volatile ("b.mov  r7, %0" :: "r" (b) : "r7");
  asm volatile ("b.mov  r6, r5");
  asm volatile ("b.mov  r8, r7");
  asm volatile ("b.andi r5, r5, 0xFFFF");
  asm volatile ("b.srli r6, r6, 16");
  asm volatile ("b.andi r7, r7, 0xFFFF");
  asm volatile ("b.srli r8, r8, 16");
  asm volatile ("b.mul  r3, r5, r8");
  asm volatile ("b.mul  r4, r5, r7");
  asm volatile ("b.mul  r5, r6, r7");
  asm volatile ("b.mul  r6, r6, r8");
  asm volatile ("b.srli r8, r3, 16");
  asm volatile ("b.slli r7, r3, 16");
  asm volatile ("b.add  r4, r4, r7");
  asm volatile ("b.addc r3, r0, r8");
  asm volatile ("b.srli r8, r5, 16");
  asm volatile ("b.slli r7, r5, 16");
  asm volatile ("b.add  r4, r4, r7");
  asm volatile ("b.addc r3, r3, r8");
  asm volatile ("b.add  r3, r3, r6");
  // reurn r3(H),r4(L)
}

#pragma GCC diagnostic pop

uint64 MulDiv::divu64u64(uint64 n, uint64 d, uint64 *r)
{
  uint64 q = (uint64)-1;
  if (d)
  {
    q = 0;
    uint32 a = System::lastBit64(n);
    uint32 b = System::lastBit64(d);
    for (b = (a > b ? a - b : 0); n >= d; --b)
    {
      uint64 m = d << b;
      if (n >= m)
      {
        n -= m;
        q |= 1ULL << b;
      }
    }
  }
  if (r)
    *r = n;
  return q;
}
