/*
  pulscounter.cpp - Puls Counter Library for NXP-JN516x

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#include "sysctrl.h"
#include "pulscounter.h"

bool PulsCounter::configure(COUNTER counter, bool falling, SAMPLES samples, COMBINE combine, PR_HWINT_APPCALLBACK prPcCallback)
{
  switch (counter)
  {
    case COUNTER_0: SysCtrl::interrupt(SysCtrl::INTSRC_PC0, prPcCallback); break;
    case COUNTER_1: SysCtrl::interrupt(SysCtrl::INTSRC_PC1, prPcCallback); break;
  }
  return bAHI_PulseCounterConfigure(counter, falling, samples, combine, prPcCallback != 0);
}

void PulsCounter::location(COUNTER counter, bool location)
{
  vAHI_PulseCounterSetLocation(counter, location);
}

bool PulsCounter::reference(COUNTER counter, uint32 count)
{
  return bAHI_SetPulseCounterRef(counter, count);
}

bool PulsCounter::start(COUNTER counter)
{
  return bAHI_StartPulseCounter(counter);
}

bool PulsCounter::stop(COUNTER counter)
{
  return bAHI_StopPulseCounter(counter);
}

bool PulsCounter::status(COUNTER counter)
{
  switch (counter)
  {
    case COUNTER_0: return u32AHI_PulseCounterStatus() & E_AHI_SYSCTRL_PC0_MASK;
    case COUNTER_1: return u32AHI_PulseCounterStatus() & E_AHI_SYSCTRL_PC1_MASK;
  }
  return false;
}
