/*
  pulscounter.h - Puls Counter Library for NXP-JN516x

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#pragma once

#include <jendefs.h>
#include <AppHardwareApi.h>


/*
   I/O Ports

      Pulse Counter 0: DIO1/DIO4
      Pulse Counter 1: DIO8/DIO5
 */

class PulsCounter
{
  public:

    typedef enum
    {
      COUNTER_0 = E_AHI_PC_0, // Pulse Counter 0 or combined counter
      COUNTER_1 = E_AHI_PC_1, // Pulse Counter 1
    } COUNTER;

    typedef enum
    {
      SAMPLES_0 = 0, // No debounce (maximum input frequency of 100kHz)
      SAMPLES_2 = 2, // 2 samples (maximum input frequency of 3.7kHz)
      SAMPLES_4 = 4, // 4 samples (maximum input frequency of 2.2kHz)
      SAMPLES_8 = 8, // 8 samples (maximum input frequency of 1.2kHz)
    } SAMPLES;

    typedef enum
    {
      COMBINE_OFF = E_AHI_PC_COMBINE_OFF, // 0 - Pulse counters not combined
      COMBINE_ON0 = E_AHI_PC_COMBINE_ON0, // 1 - Counters combined using PC0 input
      COMBINE_ON1 = E_AHI_PC_COMBINE_ON1, // 2 - Counters combined using PC1 input
    } COMBINE;

    static bool configure(COUNTER counter, bool falling, SAMPLES samples = SAMPLES_0, COMBINE combine = COMBINE_OFF, PR_HWINT_APPCALLBACK prPcCallback = 0);
    // location:
    //   TRUE  - DIO4 (Pulse Counter 0) or DIO5 (Pulse Counter 1)
    //   FALSE - DIO1 (Pulse Counter 0) or DIO8 (Pulse Counter 1)
    static void location(COUNTER counter, bool location);
    static bool reference(COUNTER counter, uint32 count);
    static bool start(COUNTER counter);
    static bool stop(COUNTER counter);
    static bool status(COUNTER counter);
};
