/*
  sysctrl.cpp - System Control Library for NXP-JN516x

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#include "system.h"
#include "sysctrl.h"

PR_HWINT_APPCALLBACK SysCtrl::_callbacks[];

void SysCtrl::begin(void)
{
  vAHI_SysCtrlRegisterCallback(SysCtrlCallback);
}

void SysCtrl::interrupt(INTSRC intsrc, PR_HWINT_APPCALLBACK prSysCtrlCallback)
{
  _callbacks[intsrc] = prSysCtrlCallback;
}

void SysCtrl::clearInterruptStatus(INTSRC intsrc)
{
  vAHI_ClearSystemEventStatus(intsrc);
}

void SysCtrl::SysCtrlCallback(uint32 u32Device, uint32 u32ItemBitmap)
{
  (void)u32Device;  // E_AHI_DEVICE_SYSCTRL
  uint32 pos;
  while ((pos = System::firstBit(u32ItemBitmap)) != 0)
  {
    uint32 bit = 1 << --pos;
    PR_HWINT_APPCALLBACK cb = _callbacks[pos];
    if (cb)
      (cb)(u32Device, bit);
    u32ItemBitmap &= ~bit;
  }
}
