/*
  system.h - System Library for NXP-JN516x

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#pragma once

#include <jendefs.h>
#include <AppHardwareApi.h>

extern void polling(void);
extern void yield(void);
extern void empty(uint32 u32Device, uint32 u32ItemBitmap);

class System
{
  public:

    static uint32 millis(void);
    static uint32 micros(void);
    static uint64 micros64(void);
    static void delayMicroseconds(uint32 us, uint8 busyLoop = 0) __attribute__((noinline,optimize(0)));
    static void delayMicroseconds64(uint64 us, uint8 busyLoop = 0) __attribute__((noinline,optimize(0)));
    static void delay(uint32 ms, uint8 busyLoop = 0) __attribute__((noinline,optimize(0)));
    static uint32 firstBit(uint32 n) __attribute__((optimize(1)));
    static uint32 firstBit64(uint64 n) __attribute__((optimize(1)));
    static uint32 lastBit(uint32 n) __attribute__((optimize(1)));
    static uint32 lastBit64(uint64 n) __attribute__((optimize(1)));
    static void enableInterrupts(void) __attribute__((optimize(1)));
    static void disableInterrupts(void) __attribute__((optimize(1)));
    static uint32 disableAndSaveInterrupts(void) __attribute__((optimize(1)));
    static void restoreInterrupts(uint32 save) __attribute__((optimize(1)));
    static void interruptPriority(uint16 u16Mask, uint8 u8Level) __attribute__((optimize(1)));
    static void cpuDoze(void) __attribute__((optimize(1)));

    typedef enum
    {
      SLEEP_OSCON_RAMON   = E_AHI_SLEEP_OSCON_RAMON,   // 32Khz Osc on and Ram On
      SLEEP_OSCON_RAMOFF  = E_AHI_SLEEP_OSCON_RAMOFF,  // 32Khz Osc on and Ram off
      SLEEP_OSCOFF_RAMON  = E_AHI_SLEEP_OSCOFF_RAMON,  // 32Khz Osc off and Ram on
      SLEEP_OSCOFF_RAMOFF = E_AHI_SLEEP_OSCOFF_RAMOFF, // 32Khz Osc off and Ram off
      SLEEP_DEEP          = E_AHI_SLEEP_DEEP,          // Deep Sleep
    } SLEEP;

    static void sleep(SLEEP mode);
    static void protocolPower(bool enable);

    typedef enum
    {
      POWER_SLEEP_WAKEUP = 1 << 0,  // Device has completed a sleep-wake cycle
      POWER_SLEEP_RAM    = 1 << 1,  // RAM contents were retained during sleep
      POWER_ANALOG       = 1 << 2,  // Analogue power domain is switched on
      POWER_PROTOCOL     = 1 << 3,  // Protocol logic is operational
      POWER_WATCHDOG     = 1 << 7,  // Watchdog caused last device restart
      POWER_CLOCK_32K    = 1 << 10, // 32kHz clock is ready
      POWER_DEEP_SLEEP   = 1 << 11, // Device has just come out of Deep Sleep mode
    } POWER;

    // bit combination
    static POWER powerStatus(void);
    static void reset(void);
    // Number of NVM location from which word is to be read: 0, 1, 2 or 3
    static void writeNVData(uint8 u8Location, uint32 u32WriteData);
    // Number of NVM location from which word is to be read: 0, 1, 2 or 3
    static uint32 readNVData(uint8 u8Location);
    static void setStackOverflow(bool_t bStkOvfEn, uint32 u32Addr);
    static void debugger(bool enable, bool location);
    static void storeDebug(uint32 *pRegStorage);
    static void restoreDebug(uint32 *pRegStorage);
};
