/*
  ticktimer.cpp - Tick Timer Library for NXP-JN516x

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#include "ticktimer.h"

volatile uint64 TickTimer::_overflow;

void TickTimer::begin(void)
{
  vAHI_TickTimerIntEnable(false);
  vAHI_TickTimerConfigure(E_AHI_TICK_TIMER_DISABLE);
  vAHI_TickTimerRegisterCallback(TickTimerCallback);
  vAHI_TickTimerInterval(0);
  vAHI_TickTimerWrite(_overflow = 0);
  vAHI_TickTimerConfigure(E_AHI_TICK_TIMER_CONT);
  vAHI_TickTimerIntPendClr();
  vAHI_TickTimerIntEnable(true);
}

uint32 TickTimer::read(void)
{
  return u32AHI_TickTimerRead();
}

uint64 TickTimer::micros64(void)
{
  uint64 h;
  uint32 l;
  do
  {
    // Check twice because the status can be confirmed even if interrupt is enabled.
    if (bAHI_TickTimerIntStatus() && bAHI_TickTimerIntStatus())
    {
      vAHI_TickTimerIntPendClr();
      TickTimerCallback(E_AHI_DEVICE_TICK_TIMER, 1);
    }
    h = _overflow;
    l = read();
  }
  while (bAHI_TickTimerIntStatus() || ((uint32)h != (uint32)_overflow));
  return (l >> 4) | (h & ~(uint64)(INTERVAL - 1));
}

void TickTimer::TickTimerCallback(uint32 u32Device, uint32 u32ItemBitmap)
{
  (void)u32Device;     // E_AHI_DEVICE_TICK_TIMER
  (void)u32ItemBitmap; // 1
  _overflow += INTERVAL >> 4;
}
