/*
  waketimer.cpp - Wake Timer Library for NXP-JN516x

  Copyright (c) 2022 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#include "sysctrl.h"
#include "muldiv.h"
#include "waketimer.h"

uint32 WakeTimer::_calibrate = 10000;

void WakeTimer::begin(void)
{
  stop(WAKE_0);
  firedStatus(WAKE_0);
  firedStatus(WAKE_1);
  _calibrate = u32AHI_WakeTimerCalibrate();
}

void WakeTimer::start(WAKE wake, uint32 seconds, PR_HWINT_APPCALLBACK ptWkCallback)
{
  uint64 u64Count = MulDiv::divu64u64(MulDiv::mulu32u32(320000000UL, seconds), _calibrate);
  if ((u64Count > 1) && (u64Count <= 0x1FFFFFFFFFF))
  {
    switch (wake)
    {
      case WAKE_0: SysCtrl::interrupt(SysCtrl::INTSRC_WK0, ptWkCallback); break;
      case WAKE_1: SysCtrl::interrupt(SysCtrl::INTSRC_WK1, ptWkCallback); break;
    }
    vAHI_WakeTimerEnable(wake, ptWkCallback != 0);
    vAHI_WakeTimerStartLarge(wake, u64Count);
  }
}

void WakeTimer::stop(WAKE wake)
{
  vAHI_WakeTimerStop(wake);
}

uint64 WakeTimer::read(WAKE wake)
{
  return u64AHI_WakeTimerReadLarge(wake);
}

uint8 WakeTimer::status(void)
{
  return u8AHI_WakeTimerStatus();
}

bool WakeTimer::status(WAKE wake)
{
  switch (wake)
  {
    case WAKE_0: return (status() & E_AHI_WAKE_TIMER_MASK_0) != 0;
    case WAKE_1: return (status() & E_AHI_WAKE_TIMER_MASK_1) != 0;
  }
  return false;
}

uint8 WakeTimer::firedStatus(void)
{
  return u8AHI_WakeTimerFiredStatus();
}

bool WakeTimer::firedStatus(WAKE wake)
{
  switch (wake)
  {
    case WAKE_0: return (firedStatus() & E_AHI_WAKE_TIMER_MASK_0) != 0;
    case WAKE_1: return (firedStatus() & E_AHI_WAKE_TIMER_MASK_1) != 0;
  }
  return false;
}

uint32 WakeTimer::calibrate(void)
{
  return _calibrate;
}
